/*
 * FreemarkerDatabase.java January 2006
 *
 * Copyright (C) 2006, Niall Gallagher <niallg@users.sf.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General 
 * Public License along with this library; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA  02111-1307  USA
 */

package simple.template.freemarker;

import freemarker.template.SimpleHash;
import simple.template.Database;
import java.util.HashSet;
import java.util.Set;

/**
 * The <code>FreemarkerDatabase</code> provides an adapter database
 * implementation for the <code>SimpleHash</code> object. This will
 * be unsynchronized and delegates to a <code>SimpleHash</code>
 * object internally, which can be specified on instantiation. 
 *
 * @author Niall Gallagher
 */
class FreemarkerDatabase implements Database {
        
   /**
    * The internal storage used by this instance.
    */
   protected SimpleHash data;
   
   /**
    * Constructor for the <code>FreemarkerDatabase</code>. This
    * will create a database without any specific inner storage. 
    * This should be used by subclasses that wish to specify the
    * storage that is used by the instance.
    */   
   protected FreemarkerDatabase() {
      super();           
   }
   
   /**
    * Constructor for the <code>FreemarkerDatabase</code>. This
    * will create a database that uses the specified object as
    * its internal storage. This should be used as an adapter for
    * the <code>freemarker.template.SimpleHash</code> object.
    *
    * @param data this contains a set of mappings to be used
    */   
   public FreemarkerDatabase(SimpleHash data) {
      this.data = data;
   }
   
   /**
    * The <code>put</code> method is used to insert a mapping in
    * the database that pairs the issued name with the issued
    * value. The value can be referenced in future by its name.
    * Properties set with this method will be set within this
    * instances data source, which is available to subclasses.
    *
    * @param name this is the name of the value being inserted
    * @param value this is the named value that is inserted
    */   
   public void put(String name, Object value) {
      try {
         data.put(name, value);
      }catch(Exception e){
         return;
      }
   }
   
   /**
    * The <code>get</code> method is used to retrieve the value
    * mapped to the specified name. If a value does not exist
    * matching the given name, then this returns null. All the 
    * property values retrieved by this method are obtained 
    * from an internal source that is accessible to subclasses.
    * 
    * @param name this is the name of the value to be retrieved
    *  
    * @return returns the value if it exists or null otherwise
    */
   public Object get(String name) {
      try {
         return data.get(name);
      }catch(Exception e){
         return null;
      }
   }
   
   /**
    * The <code>remove</code> method is used to remove the 
    * mapping from the database. This method either removes
    * the value or returns silently if the name does not exist.
    * This removes the mapping from the internal data source.
    *
    * @param name this is the name of the value to be removed
    */
   public void remove(String name) {
      try{
         data.toMap().remove(name);
      }catch(Exception e){
         return;
      }
   }
   
   /**
    * To ascertain what mappings exist, the names of all values
    * previously put into this database can be retrieved with 
    * this method. This will return a <code>Set</code> that 
    * contains the names of all the mappings added to this.
    *
    * @return this returns all the keys for existing mappings
    */
   public Set keySet(){
      try {
         return data.toMap().keySet();
      }catch(Exception e){
         return new HashSet();
      }
   }

   /**
    * The <code>contains</code> method is used to determine if
    * a mapping exists for the given name. This returns true if
    * the mapping exists or false otherwise. 
    *
    * @param name this is the name of the mapping to determine
    *
    * @return returns true if a mapping exists, false otherwise
    */
   public boolean contains(String name) {
      try {
         return data.toMap().containsKey(name);
      }catch(Exception e){
         return false;
      }
   } 
}
