/*
 * FileSource.java May 2005
 *
 * Copyright (C) 2005, Niall Gallagher <niallg@users.sf.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General 
 * Public License along with this library; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA  02111-1307  USA
 */

package simple.template.layout;

import simple.http.serve.Context;
import simple.http.serve.Content;
import java.io.PrintWriter;

/**
 * The <code>FileSource</code> is used to insert a named file as
 * a tile into a template document. This uses the value it has
 * been provided to acquire a <code>Content</code> object from
 * the <code>Context</code>. This is essentially used to create
 * a UTF-8 encoding of the files contents without parsing it.
 * 
 * @author Niall Gallagher
 *
 * @see simple.template.layout.TileLayout
 */ 
final class FileSource implements Source{

   /**
    * This is the context that is used to acquire the file.
    */         
   private Context context;

   /**
    * This is a URI path referencing the targeted file.
    */ 
   private String value;
        
   /**
    * Constructor for the <code>FileSource</code> object. This
    * is provided with the templating system context and the 
    * tile definition value used to reference the file source
    * that is to be generated by this implementation.
    * 
    * @param context the context used by the templating system
    * @param value this is the name of the file to be generated
    */ 
   public  FileSource(Context context, String value){
      this.context = context;
      this.value = value;
   }

   /**
    * This provides an object used by a <code>Tile</code> to push
    * the contents of a file into a template. The template can
    * use the provided UTF-8 encoded file as a string. This can
    * be used for convenience when it is not necessary to parse
    * a file that is to be included within the template.
    *
    * @param layout the layout used to acquire all documents
    * @param data this is the data source used by the template
    *
    * @return this returns the contents of the file from UTF-8
    */ 
   public Object getValue(Layout layout, Object data) throws Exception{
      return context.getContent(value);
   }        

   /**
    * This is used to provide an efficient means for displaying 
    * the file referenced. This will simply write the UTF-8 string
    * version of the document to the provided print writer. It is
    * up to the containing context to provide caching and other
    * such features, which enable the file to be acquired quickly.
    *
    * @param layout the layout used to acquire all documents
    * @param data this is the data source used by the template
    * @param out this is the writer to write the file text to
    */ 
   public void write(Layout layout, Object data, PrintWriter out) throws Exception{
      out.print(context.getContent(value));           
   }    
}
